//
// Harmonica Tabs Plugin
//
// Copyright (C) 2015 Be-3
//
// Based on Note Names Plugin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation and appearing in
// the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
  version: "2.0"
  description: qsTr("Griffschrift in Noten umwandeln (Club C/F)")
  menuPath: "Plugins.Griffschrift.in Noten umwandeln.von Club C/F"

  property variant col_black : "#000000"
  property variant col_push  : "#DD0000"
  property variant col_pull  : "#008800"


  // BTUTTON LAYOUT (<pitch.tpc> or 0 for non-existent keys)
  // 4th row      * * * * * * * * * *     (10)
  // 3rd row     * * * * * * * * * * *    (11)
  // 2nd row    * * * * *(*)* * * * * *   (12)
  // 1st row   * * * * * * * * * * * * *  (13)

  //// Steirische normal, 46 Tasten, G/C/F/B
  //   property variant  push1 : [43.15, 47.19, 50.16, 55.15, 59.19, 62.16, 67.15, 71.19, 74.16, 79.15, 83.19, 86.16, 91.15]
  //   property variant  push2 :     [48.14, 52.18, 55.15, 60.14, 64.18, 67.15, 72.14, 76.18, 79.15, 84.14, 88.18, 91.15]
  //   property variant  push3 :         [53.13, 57.17, 60.14, 65.13, 69.17, 72.14, 77.13, 81.17, 84.14, 89.13, 93.17]
  //   property variant  push4 :             [58.12, 62.16, 65.13, 70.12, 74.16, 77.13, 82.12, 86.16, 89.13, 94.12]
  //
  //   property variant  pull1 : [48.14, 50.16, 54.20, 57.17, 60.14, 64.18, 66.20, 69.17, 72.14, 76.18, 78.20, 81.17, 84.14]
  //   property variant  pull2 :     [53.13, 55.15, 59.19, 62.16, 65.13, 67.15, 71.19, 74.16, 77.13, 81.17, 83.19, 86.16]
  //   property variant  pull3 :         [58.12, 60.14, 64.18, 67.15, 70.12, 72.14, 76.18, 79.15, 82.12, 86.16, 88.18]
  //   property variant  pull4 :             [63.11, 65.13, 69.17, 72.14, 75.11, 77.13, 81.17, 84.14, 87.11, 91.15]


  // Hohner Club Morino, C/F
     property variant  push1 : [48.14, 52.18, 55.15, 60.14, 64.18, 67.15, 72.14, 76.18, 79.15, 84.14, 88.18, 91.15, 0]
     property variant  push2 :     [53.13, 57.17, 60.14, 65.13, 69.17, 72.14, 77.13, 81.17, 84.14, 89.13, 93.17,  0]
     property variant  push3 :         [54.20, 62.16, 63.11, 70.12, 66.20, 75.11, 74.16, 78.20, 87.11, 86.16,  0]
     property variant  push4 :              [ 0,     0,     0,     0,     0,     0,     0,     0,     0,     0]
  
     property variant  pull1 : [50.16, 55.15, 59.19, 62.16, 65.13, 69.17, 71.19, 74.16, 77.13, 81.17, 83.19, 84.14, 0]
     property variant  pull2 :     [58.12, 60.14, 64.18, 67.15, 70.12, 72.14, 76.18, 79.15, 82.12, 86.16, 88.18, 0]
     property variant  pull3 :         [56.10, 63.11, 61.21, 66.20, 68.22, 73.21, 75.11, 80.22, 85.21, 87.11, 0]
     property variant  pull4 :              [ 0,     0,     0,     0,     0,     0,     0,     0,     0,     0]

  //// Hohner Erica, C/F
  //   property variant  push1 : [ 0,    66.20, 55.15, 60.14, 64.18, 67.15, 72.14, 76.18, 79.15, 84.14, 88.18, 91.15, 0]
  //   property variant  push2 :     [0,    63.11, 60.14, 65.13, 69.17, 72.14, 77.13, 81.17, 84.14, 89.13, 93.17,  0]
  //   property variant  push3 :      [  0,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0]
  //   property variant  push4 :          [ 0,     0,     0,     0,     0,     0,     0,     0,     0,     0]
  //
  //   property variant  pull1 : [0,    67.22, 59.19, 62.16, 65.13, 69.17, 71.19, 74.16, 77.13, 81.17, 83.19, 86.16, 0]
  //   property variant  pull2 :    [0, 60.21, 64.18, 67.15, 70.12, 74.16, 76.18, 79.15, 82.12, 86.16, 88.18, 0]
  //   property variant  pull3 :      [ 0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0]
  //   property variant  pull4 :         [ 0,     0,     0,     0,     0,     0,     0,     0,     0,     0]




  function tabNotes (notes, text) {
    for (var i = 0; i < notes.length; i++) {

      var button = -1
      var row = -1
      var dir = 0
      var note = 0
      var pitch = 0
      var tpc   = 0


      if (typeof notes[i].pitch === "undefined") // just in case
      return;

      switch (notes[i].color) {
        case col_push: dir =  1; break;
        case col_pull: dir = -1; break;
        default: return;
      }

      if (notes[i].tpc >= 13 && notes[i].tpc <= 19) {
        button = Math.floor ( notes[i].pitch / 1.7 ) - 30;
        if ( button % 2 == 0 )
        row = 1
        else
        row = 2;
      }
      else {
        if (notes[i].tpc >= 20 && notes[i].tpc <= 26)
        if (notes[i].pitch < 75)
        button = Math.floor ( notes[i].pitch / 1.82 ) - 30
        else
        button = Math.floor ( notes[i].pitch / 1.8 ) - 30;
        if (notes[i].tpc >= 27 && notes[i].tpc <= 33)
        button = Math.floor ( notes[i].pitch / 1.75 ) - 32;

        if ( button % 2 == 0 )
        row = 3
        else
        row = 4;
      }
      button = Math.floor ( button / 2 );

      // jetzt haben wir button und row.
      // Test mit DRUCK:
      if ( button > 13 - row ) {
        console.log("Button " + button + " in row " + row + " too high!");
        text.text = "?";
        return;
      }
      if ( button < 0 ) {
        console.log("Button " + button + " in row " + row + " too low!");
        text.text = "?";
        return;
      }
      if (dir == 1)   // PUSH
      switch (row) {
        case 1: note = push1[button]; break;
        case 2: note = push2[button]; break;
        case 3: note = push3[button]; break;
        case 4: note = push4[button]; break;
        default: note = 0; break;
      }
      else        // PULL
      switch (row) {
        case 1: note = pull1[button]; break;
        case 2: note = pull2[button]; break;
        case 3: note = pull3[button]; break;
        case 4: note = pull4[button]; break;
        default: note = 0; break;
      }


      if (note == 0) {
        console.log("Button " + button + " in row " + row + "undefined!");
        text.text = "?";
        return;
      }

      pitch = Math.floor( note );
      tpc   = Math.round(100 * (note - pitch));
      notes[i].accidentalType = 0;
      notes[i].pitch = pitch;
      notes[i].tpc1  = tpc;
      notes[i].tpc2  = tpc;
    } // end for note
  }

  onRun: {
    if (typeof curScore === 'undefined')
    Qt.quit();
    var cursor = curScore.newCursor();
    var startStaff;
    var endStaff;
    var endTick;
    var fullScore = false;
    cursor.rewind(1);
    if (!cursor.segment) {
      // no selection
      fullScore = true;
      startStaff = 0; // start with 1st staff
      endStaff = curScore.nstaves - 1; // and end with last
    } else {
      startStaff = cursor.staffIdx;
      cursor.rewind(2);
      if (cursor.tick == 0) {
        // this happens when the selection includes
        // the last measure of the score.
        // rewind(2) goes behind the last segment (where
        // there's none) and sets tick=0
        endTick = curScore.lastSegment.tick + 1;
      } else {
        endTick = cursor.tick;
      }
      endStaff = cursor.staffIdx;
    }
    console.log(startStaff + " - " + endStaff + " - " + endTick)

    for (var staff = startStaff; staff <= endStaff; staff++) {
      for (var voice = 0; voice < 4; voice++) {
        cursor.rewind(1); // beginning of selection
        cursor.voice = voice;
        cursor.staffIdx = staff;

        if (fullScore) // no selection
        cursor.rewind(0); // beginning of score

        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
          if (cursor.element && cursor.element.type == Element.CHORD) {
            var text = newElement(Element.STAFF_TEXT);

            var graceChords = cursor.element.graceNotes;
            for (var i = 0; i < graceChords.length; i++) {
              // iterate through all grace chords
              var notes = graceChords[i].notes;
              tabNotes(notes, text);
              // there seems to be no way of knowing the exact horizontal pos.
              // of a grace note, so we have to guess:
              text.pos.x = -2.5 * (graceChords.length - i);
              switch (voice) {
                case 0: text.pos.y = -1; break;
                case 1: text.pos.y = 10; break;
                case 2: text.pos.y = -2; break;
                case 3: text.pos.y = 12; break;
              }

              cursor.add(text);
              // new text for next element
              text = newElement(Element.STAFF_TEXT);
            }

            var notes = cursor.element.notes;
            tabNotes(notes, text);

            switch (voice) {
              case 0: text.pos.y = -1; break;
              case 1: text.pos.y = 10; break;
              case 2: text.pos.y = -2; break;
              case 3: text.pos.y = 12; break;
            }
            if ((voice == 0) && (notes[0].pitch > 83))
            text.pos.x = 1;
            cursor.add(text);
          } // end if CHORD
          cursor.next();
        } // end while segment
      } // end for voice
    } // end for staff
    Qt.quit();
  } // end onRun
}